// NURBSDoc.cpp : implementation of the CNURBSDoc class
//

#include "stdafx.h"
#include "NURBS.h"

#include "NURBSDoc.h"
#include "EditUzelDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CNURBSDoc

IMPLEMENT_DYNCREATE(CNURBSDoc, CDocument)

BEGIN_MESSAGE_MAP(CNURBSDoc, CDocument)
	//{{AFX_MSG_MAP(CNURBSDoc)
	ON_COMMAND(ID_INSERT_UZLY, OnInsertUzly)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CNURBSDoc construction/destruction

CNURBSDoc::CNURBSDoc()
{
}

CNURBSDoc::~CNURBSDoc()
{
}

/////////////////////////////////////////////////////////////////////////////
// CNURBSDoc serialization

void CNURBSDoc::Serialize(CArchive& ar)
{
	int i, pocet, hodnota;
	CPoint point;

	if (ar.IsStoring())
	{
		pocet = m_body.GetSize();
		ar << (LONG) pocet;
		for (i = 0; i < pocet; i++) {
			ar << m_body.GetAt(i);
			ar << m_vahy.GetAt(i);
		}
		
		pocet += m_stupen + 1;
		ar << (LONG) pocet;
		for (i = 0; i < pocet; i++)
			ar << m_uzly.GetAt(i);
		
		ar << (LONG) m_stupen;
	}
	else
	{
		ar >> (int) pocet;
		for (i = 0; i < pocet; i++) {
			ar >> point;
			m_body.Add(point);
			ar >> (int) hodnota;
			m_vahy.Add(hodnota);
		}
		
		ar >> (int) pocet;
		for (i = 0; i < pocet; i++) {
			ar >> (int) hodnota;
			m_uzly.Add(hodnota);
		}

		ar >> (int) m_stupen;
	}
}

/////////////////////////////////////////////////////////////////////////////
// CNURBSDoc diagnostics

#ifdef _DEBUG
void CNURBSDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CNURBSDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CNURBSDoc commands

void CNURBSDoc::DeleteContents() 
{
	m_body.RemoveAll();
	m_vahy.RemoveAll();
	m_uzly.RemoveAll();

	CDocument::DeleteContents();
}

void CNURBSDoc::OnInsertUzly() 
{
	CEditUzelDlg dlg;
	dlg.m_index = 1;
	dlg.m_pocet = m_stupen + m_body.GetSize() + 1;
	dlg.m_uzel = m_uzly.GetAt(0);
	dlg.uzly.Append(m_uzly);
	if (dlg.DoModal() == IDOK) {
		m_uzly.RemoveAll();
		m_uzly.Append(dlg.uzly);
		UpdateAllViews(NULL);
	}

}

/////////////////////////////////////////////////////////////////////////////

double CNURBSDoc::BaseFunc(int i, int p, double u)
{
	if (p == 0) {
		if (u >= m_uzly.GetAt(i) && u < m_uzly.GetAt(i + 1))
				return 1.0;
		else	return 0.0;
	}
	else {
		double	ui		= (double) m_uzly.GetAt(i),
				uip1	= (double) m_uzly.GetAt(i + p + 1),
				u1		= (double) m_uzly.GetAt(i + p) - ui,
				u2		= uip1 - (double) m_uzly.GetAt(i + 1);
		
		if (u1 != 0.0)	u1 = ((u - ui) / u1) * BaseFunc(i, p - 1, u); 
		
		if (u2 != 0.0)	u2 = ((uip1 - u) / u2) * BaseFunc(i + 1, p - 1, u);

		return	u1 + u2;
	}
}


BOOL CNURBSDoc::OnNewDocument() 
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	m_stupen = 3;
	int i;
	for (i = 0; i < (m_stupen + 1); i++)
		m_uzly.Add(0);

	return TRUE;
}
