// NURBSView.cpp : implementation of the CNURBSView class
//

#include "stdafx.h"
#include "NURBS.h"

#include "NURBSDoc.h"
#include "NURBSView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CNURBSView

IMPLEMENT_DYNCREATE(CNURBSView, CView)

BEGIN_MESSAGE_MAP(CNURBSView, CView)
	//{{AFX_MSG_MAP(CNURBSView)
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_WM_RBUTTONDOWN()
	ON_COMMAND(ID_DRAW_GRAF, OnDrawGraf)
	ON_UPDATE_COMMAND_UI(ID_DRAW_GRAF, OnUpdateDrawGraf)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CNURBSView construction/destruction

CNURBSView::CNURBSView()
{
	modra = RGB(0, 0, 255);
	cervena = RGB(255, 0, 0);
	cervene.CreatePen(PS_SOLID, 3, cervena);
	index = -1;
	draw_graf = TRUE;
}

CNURBSView::~CNURBSView()
{
}

BOOL CNURBSView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CNURBSView drawing

void CNURBSView::OnDraw(CDC* pDC)
{
	CNURBSDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	int i, n = pDoc->m_body.GetSize();
	CPoint point;
	char number[3];
	int d = 5, c = 3;
	int Qx, Qy, Qxold, Qyold;
	
	for (i = 0; i < n; i++) {
		point = pDoc->m_body.GetAt(i);
		if (i != 0)
			pDC->LineTo(point);
		pDC->Ellipse(point.x - 5, point.y + 5, point.x + 5, point.y - 5);
		sprintf(number, "%3d", i + 1);
		pDC->TextOut(point.x + 5, point.y + 5, number, 3);
		pDC->MoveTo(point);
	}

	if (draw_graf && n >= (pDoc->m_stupen + 1)) {

		int first = pDoc->m_stupen,
			last = first + n - 3;

		pDC->SelectObject(&cervene);

		for (i = first; i < last; i++) {

			double	umin = (double) pDoc->m_uzly.GetAt(i),
					umax = (double) pDoc->m_uzly.GetAt(i + 1),
					Bi3, Bi2, Bi1, Bi0,
					u;

			if (umin >= umax) continue;
			u = umin;
			while (true) {
				Bi3 = pDoc->BaseFunc(i - 3, pDoc->m_stupen, u);
				Bi2 = pDoc->BaseFunc(i - 2, pDoc->m_stupen, u);
				Bi1 = pDoc->BaseFunc(i - 1, pDoc->m_stupen, u);
				Bi0 = pDoc->BaseFunc(i, pDoc->m_stupen, u);
				Qx = (int) (Bi3 * pDoc->m_body.GetAt(i - 3).x +
							Bi2 * pDoc->m_body.GetAt(i - 2).x +
							Bi1 * pDoc->m_body.GetAt(i - 1).x +
							Bi0 * pDoc->m_body.GetAt(i).x		);
				Qy = (int) (Bi3 * pDoc->m_body.GetAt(i - 3).y +
							Bi2 * pDoc->m_body.GetAt(i - 2).y +
							Bi1 * pDoc->m_body.GetAt(i - 1).y +
							Bi0 * pDoc->m_body.GetAt(i).y		);
				
				if (u == umin) {
						pDC->MoveTo(Qx, Qy);
						pDC->Ellipse(Qx - d, Qy + d, Qx + d, Qy - d);
						pDC->Ellipse(Qx - c, Qy + c, Qx + c, Qy - c);
				}
				else	if (Qx != Qxold || Qy != Qyold) {
							pDC->LineTo(Qx, Qy);
				}
				Qxold = Qx;
				Qyold = Qy;

				if (u == umax - .0001) break;
				u += .01;
				if (u > umax)
					u = umax - .0001;
				continue;
			}
			pDC->Ellipse(Qx - d, Qy + d, Qx + d, Qy - d);
			pDC->Ellipse(Qx - c, Qy + c, Qx + c, Qy - c);
		}
	}		

}

/////////////////////////////////////////////////////////////////////////////
// CNURBSView printing

BOOL CNURBSView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	pInfo->SetMaxPage(1);
	return DoPreparePrinting(pInfo);
}

void CNURBSView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CNURBSView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CNURBSView diagnostics

#ifdef _DEBUG
void CNURBSView::AssertValid() const
{
	CView::AssertValid();
}

void CNURBSView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CNURBSDoc* CNURBSView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CNURBSDoc)));
	return (CNURBSDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CNURBSView message handlers

void CNURBSView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CClientDC dc(this);
	OnPrepareDC(&dc);
	
	CNURBSDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	int i;
	CPoint bod;
	CRect rect;
	for (i = 0; i < pDoc->m_body.GetSize(); i++) {
		bod = pDoc->m_body.GetAt(i);
		dc.LPtoDP(&bod);
		rect = CRect(bod.x - 5, bod.y + 5, bod.x + 5, bod.y - 5);
		rect.NormalizeRect();
		if (rect.PtInRect(point)) {
			index = i;
			SetCapture();
			return;
		}
	}
	dc.DPtoLP(&point);
	pDoc->m_body.Add(point);
	pDoc->m_vahy.Add(1);
	pDoc->m_uzly.Add(0);
	RedrawWindow();
}

void CNURBSView::OnPrepareDC(CDC* pDC, CPrintInfo* pInfo) 
{
	pDC->SetMapMode(MM_LOMETRIC);
	pDC->SetTextColor(modra);
	CView::OnPrepareDC(pDC, pInfo);
}

void CNURBSView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	ReleaseCapture();
}

void CNURBSView::OnMouseMove(UINT nFlags, CPoint point) 
{
	if (GetCapture() != this)
		return;

	CClientDC dc(this);
	OnPrepareDC(&dc);
	dc.DPtoLP(&point);

	CNURBSDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	pDoc->m_body.SetAt(index, point);
	RedrawWindow();

}

void CNURBSView::OnRButtonDown(UINT nFlags, CPoint point) 
{
	CClientDC dc(this);
	OnPrepareDC(&dc);
	
	CNURBSDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	int i;
	CPoint bod;
	CRect rect;
	for (i = 0; i < pDoc->m_body.GetSize(); i++) {
		bod = pDoc->m_body.GetAt(i);
		dc.LPtoDP(&bod);
		rect = CRect(bod.x - 5, bod.y + 5, bod.x + 5, bod.y - 5);
		rect.NormalizeRect();
		if (rect.PtInRect(point)) {
			pDoc->m_body.RemoveAt(i);
			pDoc->m_vahy.RemoveAt(i);
			pDoc->m_uzly.RemoveAt(pDoc->m_uzly.GetSize() - 1);
			/*if (pDoc->m_body.GetSize() < (pDoc->m_stupen + 1))
				draw_graf = FALSE;*/
			RedrawWindow();
		}
	}
}


void CNURBSView::OnDrawGraf() 
{
/*	CNURBSDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	if (pDoc->m_body.GetSize() >= (pDoc->m_stupen + 1)) {*/
		draw_graf = !draw_graf;
		RedrawWindow();
/*	}
	else {
		draw_graf = FALSE;
		AfxMessageBox("Nen co kreslit, zadej vce bod!");
	}*/
}

void CNURBSView::OnUpdateDrawGraf(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(draw_graf);	
}

